-- Load the Lua/APR binding.
local apr = require 'apr'

-- Initialize the server socket.
local server = assert(apr.socket_create())
assert(server:bind('*', 1234))
assert(server:listen(10))

-- Create the poll set.
local pollset = assert(apr.pollset(10))
assert(pollset:add(server, 'input'))

-- Keep a list of connected clients around.
local connected = {}

local function broadcast(message, exclude)
  for _, client in ipairs(connected) do
    if client ~= exclude then
      assert(client:write(message, '\n'))
    end
  end
end

while true do
  -- Process incoming connections/messages.
  local status, readable, errmsg, errcode = pcall(pollset.poll, pollset, -1)
  print(readable, errrmsg, errcode)
  for _, socket in ipairs(readable) do
    if socket == server then
      -- Accept incoming connection request.
      local client = assert(server:accept())
      assert(pollset:add(client, 'input'))
      table.insert(connected, client)
      -- Announce arrival of new guest.
      broadcast("There are now " .. #connected .. " guests in the chatroom.")
    else
      -- Accept incoming chat message and broadcast it.
      broadcast(tostring(socket) .. ": " .. assert(socket:read()), socket)
    end
  end
end
